@@ -1,4 +1,4 @@
-from labelspark import connector, check_pyspark
+from labelspark import connector
 from pyspark.sql.dataframe import DataFrame
 from labelbox import Client as labelboxClient
 from labelbase import Client as labelbaseClient
@@ -27,7 +27,7 @@ class Client:
     def __init__(self, lb_api_key=None, lb_endpoint='https://api.labelbox.com/graphql', lb_enable_experimental=False, lb_app_url="https://app.labelbox.com"):
         self.lb_client = labelboxClient(lb_api_key, endpoint=lb_endpoint, enable_experimental=lb_enable_experimental, app_url=lb_app_url)
         self.base_client = labelbaseClient(lb_api_key, lb_endpoint=lb_endpoint, lb_enable_experimental=lb_enable_experimental, lb_app_url=lb_app_url)
-        check_pyspark()
+        connector.check_pyspark()
     
     def create_data_rows_from_table(
         self, table:DataFrame, lb_dataset:labelboxDataset, row_data_col:str, global_key_col:str="", external_id_col:str="",
@@ -89,27 +89,29 @@ class Client:
         
         return {"upload_results" : upload_results, "conversion_errors" : conversion_errors}
 
-    def create_table_from_dataset(self, lb_client, lb_dataset, metadata_index={}):
+    def create_table_from_dataset(self, lb_dataset:labelboxDataset, metadata_index:dict={}, verbose:bool=False, divider:str="///"):
         """ Creates a Spark Table from a Labelbox dataset, optional metadata_fields will create 1 column per field name in list
         Args:
-            lb_client           :   Required( labelbox.client.Client) - Labelbox Client object
-            lb_dataset          :   Required (labelbox.schema.dataset.Dataset) - Labelbox dataset to create a spark table from
-            metadata_index      :   Optional (dict) - Determines what metadata gets uploaded to Labelbox - dictionary where {key=column_name : value=metadata_type} - metadata_type must be one of "enum", "string", "datetime" or "number"
+            lb_dataset      :   Required (labelbox.schema.dataset.Dataset) - Labelbox dataset to create a spark table from
+            metadata_index  :   Optional (dict) - Dictionary where {key=column_name : value=metadata_type}
+                                    metadata_type must be either "enum", "string", "datetime" or "number"
+            verbose         :   Required (bool) - If True, prints details about code execution; if False, prints minimal information
+            divider         :   Optional (str) - String delimiter for all name keys generated for parent/child schemas            
         Returns:
             Spark Table - pyspark.sql.dataframe.Dataframe objet with columns "data_row_id", "global_key", "external_id", "row_data" and optional metadata fields as columns
         """        
-        spark_table = connector.sync_metadata_fields(
-            lb_client=lb_client, spark_table=None, metadata_index=metadata_index
-        )
-        if not spark_table:
-            return None
+        table = self.base_client.sync_metadata_fields(
+            table=False, get_columns_function=connector.get_columns_function, add_column_function=connector.add_column_function, 
+            get_unique_values_function=connector.get_unique_values_function, metadata_index=metadata_index, verbose=verbose
+        )                         
         starttime = datetime.now()
         metadata_fields = list(metadata_index.keys())
         data_rows_list = list(lb_dataset.export_data_rows(include_metadata=True)) if metadata_fields else list(lb_dataset.export_data_rows(include_metadata=False))
         endtime = datetime.now()
-        print(f'Labelbox data row export complete\n Start Time: {starttime}\n End Time: {endtime}\n Export Time: {endtime-starttime}\nData rows to create in table: {len(data_rows_list)}')
+        if verbose:
+            print(f'Labelbox data row export complete\n Start Time: {starttime}\n End Time: {endtime}\n Export Time: {endtime-starttime}\nData rows to create in table: {len(data_rows_list)}')
         starttime = datetime.now()
-        metadata_schema_to_name_key = connector.get_metadata_schema_to_name_key(lb_client.get_data_row_metadata_ontology())
+        metadata_schema_to_name_key = self.base_client.get_metadata_schema_to_name_key()
         # Create empty dict where {key= column name : value= list of values for column} (in order)
         table_dict = {"data_row_id" : [], "global_key" : [], "external_id" : [], "row_data" : []}
         if metadata_fields:
@@ -132,37 +134,38 @@ class Client:
                             metadata_dict[metadata_field['name']] = metadata_schema_to_name_key[metadata_field['value']].split("///")[1] if metadata_field['value'] in metadata_schema_to_name_key else metadata_field['value']
                 for metadata_name in metadata_dict:
                     table_dict[metadata_name].append(metadata_dict[metadata_name])
-        spark_table = pd.DataFrame.from_dict(table_dict).to_spark()
-        spark_table = spark_table.drop(col("lb_integration_source"))
+        table = pd.DataFrame.from_dict(table_dict).to_spark()
+        table = table.drop(col("lb_integration_source"))
         endtime = datetime.now()
-        print(f'Success: Created table from Labelbox dataset with ID {lb_dataset.uid}\n Start Time: {starttime}\n End Time: {endtime}\n Total Time: {endtime-starttime}\nData rows in table: {len(data_rows_list)}')
-        return spark_table
+        if verbose:
+            print(f'Success: Created table from Labelbox dataset with ID {lb_dataset.uid}\n Start Time: {starttime}\n End Time: {endtime}\n Total Time: {endtime-starttime}\nData rows in table: {len(data_rows_list)}')
+        return table
     
-    def upsert_table_metadata(self, lb_client, spark_table, global_key_col, global_keys_list=[], metadata_index={}):
+    def upsert_table_metadata(self, table, global_key_col, global_keys_list=[], metadata_index={}, verbose:bool=False):
         """ Upserts a Spark Table based on the most recent metadata in Labelbox, only upserts columns provided via metadata_fields list
         Args:
-            lb_client           :   Required( labelbox.client.Client) - Labelbox Client object            
-            spark_table         :   Required (pyspark.sql.dataframe.Dataframe) - Spark Table
-            lb_dataset          :   Required (labelbox.schema.dataset.Dataset) - Labelbox dataset to add data rows to
-            global_key_col      :   Required (str) - Global key column name to map Labelbox data rows to the Databricks spark table rows
-            global_keys_list    :   Optional (list) - List of global keys you wish to upsert - defaults to the whole table
-            metadata_index      :   Optional (dict) - Determines what metadata gets upserted in Databricks - dictionary where {key=column_name : value=metadata_type} - metadata_type must be one of "enum", "string", "datetime" or "number"
+            table           :   Required (pyspark.sql.dataframe.DataFrame) - Spark Table
+            global_key_col  :   Required (str) - Column name containing the data row global key
+            global_keys_list:   Optional (list) - List of global keys you wish to upsert - defaults to the whole table
+            metadata_index  :   Optional (dict) - Dictionary where {key=column_name : value=metadata_type}
+                                    metadata_type must be either "enum", "string", "datetime" or "number"
         Returns:
             Upserted Spark table
         """
         # Export data row metadata from Labelbox through global keys
         starttime = datetime.now()
-        lb_mdo = lb_client.get_data_row_metadata_ontology()
-        metadata_schema_to_name_key = connector.get_metadata_schema_to_name_key(lb_mdo)
+        lb_mdo = self.lb_client.get_data_row_metadata_ontology()
+        metadata_schema_to_name_key = self.base_client.get_metadata_schema_to_name_key(lb_mdo=lb_mdo)
         ## Either use global_keys provided or all the global keys in the provided global_key column 
-        global_keys = global_keys_list if global_keys_list else connector.get_unique_values(spark_table, global_key_col)
+        global_keys = global_keys_list if global_keys_list else connector.get_unique_values(table, global_key_col)
         ## Grab data row IDs with global_key list
-        data_row_ids = lb_client.get_data_row_ids_for_global_keys(global_keys)['results']
+        data_row_ids = self.lb_client.get_data_row_ids_for_global_keys(global_keys)['results']
         data_row_id_to_global_key = {str(data_row_ids[i]) : str(global_keys[i]) for i in range(0, len(data_row_ids))}
         ## Get data row metadata with list of data row IDs
         data_row_metadata = lb_mdo.bulk_export(data_row_ids)
         endtime = datetime.now()
-        print(f'Labelbox metadata export complete\n Start Time: {starttime}\n End Time: {endtime}\n Export Time: {endtime-starttime}\nData rows in export: {len(data_row_metadata)}')        
+        if verbose:
+            print(f'Labelbox metadata export complete\n Start Time: {starttime}\n End Time: {endtime}\n Export Time: {endtime-starttime}\nData rows in export: {len(data_row_metadata)}')        
         # Run the table through a UDF that will update metadata column values using the Global Key as the primary key        
         starttime = datetime.now()
         ## Create a dict where {metadata_field_name (column name) : {global_key : new_metadata_value}} (for enums, is the schema ID)
@@ -181,9 +184,10 @@ class Client:
         ## For each metadata field column, use a UDF upsert column values with the values in your dict where {key=global_key : value=new_metadata_value}
         upsert_udf = connector.column_upsert_udf()
         for metadata_col in upsert_dict:
-            spark_table = spark_table.withColumn(metadata_col, upsert_udf(lit(json.dumps(upsert_dict[metadata_col])), global_key_col, metadata_col))
+            table = table.withColumn(metadata_col, upsert_udf(lit(json.dumps(upsert_dict[metadata_col])), global_key_col, metadata_col))
         endtime = datetime.now()
-        print(f'Upsert table metadata complete\n Start Time: {starttime}\n End Time: {endtime}\n Total Time: {endtime-starttime}\nData rows upserted: {len(data_row_metadata)}') 
+        if verbose:
+            print(f'Upsert table metadata complete\n Start Time: {starttime}\n End Time: {endtime}\n Total Time: {endtime-starttime}\nData rows upserted: {len(data_row_metadata)}') 
         return spark_table
 
     def upsert_labelbox_metadata(self, lb_client, spark_table, global_key_col, global_keys_list=[], metadata_index={}):
@@ -200,13 +204,13 @@ class Client:
         """
         # Export data row metadata from Labelbox through global keys        
         starttime = datetime.now()
-        lb_mdo = lb_client.get_data_row_metadata_ontology()
-        metadata_schema_to_name_key = connector.get_metadata_schema_to_name_key(lb_mdo)
-        metadata_name_key_to_schema = connector.get_metadata_schema_to_name_key(lb_mdo, invert=True)
+        lb_mdo = self.lb_client.get_data_row_metadata_ontology()
+        metadata_schema_to_name_key = self.base_client.get_metadata_schema_to_name_key(lb_mdo=lb_mdo)
+        metadata_name_key_to_schema = self.base_client.get_metadata_schema_to_name_key(lb_mdo=lb_mdo, invert=True)
         ## Either use global_keys provided or all the global keys in the provided global_key column 
         global_keys = global_keys_list if global_keys_list else connector.get_unique_values(spark_table, global_key_col)
         ## Grab data row IDs with global_key list
-        data_row_ids = lb_client.get_data_row_ids_for_global_keys(global_keys)['results']
+        data_row_ids = self.lb_client.get_data_row_ids_for_global_keys(global_keys)['results']
         data_row_id_to_global_key = {str(data_row_ids[i]) : str(global_keys[i]) for i in range(0, len(data_row_ids))}
         ## Get data row metadata with list of data row IDs
         data_row_metadata = lb_mdo.bulk_export(data_row_ids)  
